#***************************************************************************
# Description
#
# 1. Batch Merging of Files for the purpose of retargeting the animation.
#
# 2. Meant to grab animation data from files that has been saved as 
#    animation only.
#
# 3. It will open file, load in the data
#
# 4. This will describe the process of grabbing a folder of files, and 
#    working with FBApplication and FBFbxOptions to load in scene data.
#***************************************************************************
# Assumptions:There will be one character per file.
#
#
#***************************************************************************
# Directions
#
# 1. Execute the script.
#
# 2. When the script prompts you to choose a target file, select a file with a  
#    character without any animation on it. This will be the character you wish
#    to merge animation onto.
#
# 3. When the script prompts you to choose a source folder, select a folder with 
#    animation only files in it. These will be the files you wish to merge 
#    onto the already open scene.
#***************************************************************************


import os
import re
from pyfbsdk import *

# Creating all the Constructors
lFp = FBFolderPopup()
lFilep = FBFilePopup()
lSystem = FBSystem()
lScene = lSystem.Scene
lApplication = FBApplication()
lPlotOptions = FBPlotOptions()

fbxList = []

#Part A: Opening the Character for which you would like to plot animation to
lFilep.Caption = "Target File: Select a file which contains a character in it"
lFilep.Style = FBFilePopupStyle.kFBFilePopupOpen
lFilep.Filter = "*"

# Set the default path.
lFilep.Path = r"C:\WORK\Project\2012 MotionBuilder Webcast Training\Naiqi\Instructor\Lesson_6_Characters\Assignment Scene Files\Assignment Scene Files\Character"

# Get the GUI to show.
lFileRes = lFilep.Execute()

# If we select files, show them, otherwise indicate that the selection was canceled.
if not lFileRes:
   print "Selection Canceled"
else:
    #Actually open the selected file into the scene
    lApplication.FileOpen(lFilep.FullFilename)

    # Part B : Select Directory you would like to merge your FBX file in from 
    # Create the popup and set necessary initial values.
    lFp.Caption = "Source Files: Select the folder containing the animation you would like to merge to the target file"

    # Set the default file path.
    lFp.Path = r"C:\WORK\Project\2012 MotionBuilder Webcast Training\Naiqi\Instructor\Lesson_6_Characters\Assignment Scene Files\Assignment Scene Files\Animations"

    # Get the GUI to show.
    lRes = lFp.Execute()

    # If you select a folder, show its name, otherwise indicate that the selection was canceled.
    if not lRes:
           print "Selection Canceled"
    else:
        # Part C: Getting the names of the files in your previously selected folder
        # Using os to get the file names from the specified folder (above) and storing names of files in a list
        fileList = os.listdir(lFp.Path)

        # Setting the regular expression to only look for .fbx extenstion
        fbxRE = re.compile('^\w+.fbx$', re.I)

        # Removing any files that do not have an .fbx extenstion
        for fname in fileList:
            mo = fbxRE.search(fname)
            if mo:
                fbxList.append(fname)    

            # Part D : Merging in all the files 
            # Verify there is a character in the scene
            if not lScene.Characters:
                print "Cannot continue at there is no character in the scene..."
            else:
                # Storing Primary Character name in a variable to use for later
                # Assumption the first character in the scene will be the primary character
                primaryCharacter = lScene.Characters[0].Name

        # Part E : Calling the Load command to merge the file in one at a time
        # Start of Merge of Files
        count = 1
        for fname in fbxList:
            print "Loading File %s..." % (fname)
            
            # Create FBFbxOptions instance and set load options
            lFbxOptions = FBFbxOptions(True)
            lFbxOptions.ActorFaces = FBElementAction.kFBElementActionAppend
            lFbxOptions.Actors = FBElementAction.kFBElementActionAppend
            lFbxOptions.Audio = FBElementAction.kFBElementActionDiscard
            lFbxOptions.Cameras = FBElementAction.kFBElementActionDiscard
            lFbxOptions.CharacterExtensions = FBElementAction.kFBElementActionAppend
            lFbxOptions.CharacterFaces = FBElementAction.kFBElementActionAppend
            lFbxOptions.Characters = FBElementAction.kFBElementActionAppend
            lFbxOptions.Constraints = FBElementAction.kFBElementActionAppend
            lFbxOptions.Devices = FBElementAction.kFBElementActionAppend
            lFbxOptions.Groups = FBElementAction.kFBElementActionAppend
            lFbxOptions.Lights = FBElementAction.kFBElementActionDiscard
            lFbxOptions.Materials = FBElementAction.kFBElementActionDiscard
            lFbxOptions.Models = FBElementAction.kFBElementActionAppend
            lFbxOptions.Notes = FBElementAction.kFBElementActionAppend
            lFbxOptions.Poses = FBElementAction.kFBElementActionAppend
            lFbxOptions.Scripts = FBElementAction.kFBElementActionDiscard
            lFbxOptions.Sets = FBElementAction.kFBElementActionDiscard
            lFbxOptions.Shaders = FBElementAction.kFBElementActionDiscard
            lFbxOptions.Story = FBElementAction.kFBElementActionAppend
            lFbxOptions.Textures = FBElementAction.kFBElementActionDiscard
            lFbxOptions.Video = FBElementAction.kFBElementActionDiscard
            lFbxOptions.ActorFacesAnimation = True
            lFbxOptions.CamerasAnimation = True
            lFbxOptions.CharacterFacesAnimation = True
            lFbxOptions.CharactersAnimation = True
            lFbxOptions.ConstraintsAnimation = True
            lFbxOptions.DevicesAnimation = False
            lFbxOptions.LightsAnimation = False
            lFbxOptions.MaterialsAnimation = False
            lFbxOptions.ModelsAnimation = True
            lFbxOptions.NotesAnimation = True
            lFbxOptions.ShadersAnimation = False
            lFbxOptions.TexturesAnimation = False
            lFbxOptions.TransportSettings = True
	 		
            # Merge files in one at a time
            lMergeResult = lApplication.FileMerge(lFp.Path + "\\" + fname,False,lFbxOptions)

            
            # To ensure the merge was successful
            if not lMergeResult:
                print "Load of filname: " + fname + " was unsuccessful..."
            else:
                    print "Load of filname: " + fname + " was successful..."
                    
                    # Module 6 ADDED CODE || START
                    # Retarget data to start character 
                    lScene.Characters[0].InputCharacter = lScene.Characters[count]
                    lScene.Characters[0].InputType = FBCharacterInputType.kFBCharacterInputCharacter
                    lScene.Characters[0].ActiveInput = True
                    
                    # Plot data to start character's control rig
                    lPlotOptions.PlotAllTakes = True
                    lScene.Characters[0].PlotAnimation ( FBCharacterPlotWhere.kFBCharacterPlotOnControlRig, lPlotOptions )
        
        count = count + 1
            
        # Part K: Saved as a new file 
        saving = FBApplication()
        saving.FileSave("NEW_CharactersMerge.fbx")

# Part L: Cleanup 
    del (fname, mo, fileList,lMergeResult, fbxRE, saving, lRes)

# Part M: Cleanup
del( lSystem, lScene, lApplication,lFp, lFilep, lFileRes, FBFilePopup, FBFolderPopup, FBFilePopupStyle, FBApplication, FBSystem, FBElementAction, FBCharacterInputType, FBFbxOptions, FBPlotOptions, FBCharacterPlotWhere, FBCharacter, FBBodyNodeId )



