from pyfbsdk import *

#------------------------------------------
# This function adds a marker to the passed MarkerSet
def addMarkerToActor ( mSet, markerName, bodyPart ):
    myMarker = FBFindModelByName(markerName)
    
    bodyId = getattr(FBSkeletonNodeId, bodyPart)
    mSet.AddMarker(bodyId,myMarker)
    
#------------------------------------------

#create an actor and set the reference model
myActor = FBActor("SampleActor")

#create and attach a marker set
myActor.MarkerSet = FBMarkerSet("SampleActor's marker set")
myMarkerSet = myActor.MarkerSet

reference = FBFindModelByName("optical")
myMarkerSet.SetReferenceModel(reference)


#we will create a dictionnary with the name of the markers and the corresponding bodyID
#first item in set is called 'key', second item is called the 'value' separate with a colon ':'
#simply change the name of the model (for example, HipsModel1) with the name of the object in your scene
#add or remove key/value if you need more or less optical model (example: "HipsModel3":'kFBSkeletonHipsIndex'...)
keyValueList = {"HipsModel1":'kFBSkeletonHipsIndex',"HipsModel2":'kFBSkeletonHipsIndex',
           "ChestModel1":'kFBSkeletonChestIndex',"ChestModel2":'kFBSkeletonChestIndex',
           "RightCollarModel1":'kFBSkeletonRightCollarIndex', 
           "RightShoulderModel1":'kFBSkeletonRightShoulderIndex', "RightShoulderModel2":'kFBSkeletonRightShoulderIndex',
           "RightElbowModel1":'kFBSkeletonRightElbowIndex', "RightElbowModel2":'kFBSkeletonRightElbowIndex', 
           "RightWristModel1":'kFBSkeletonRightWristIndex',
           "LeftCollarModel1":'kFBSkeletonLeftCollarIndex', 
           "LeftShoulderModel1":'kFBSkeletonLeftShoulderIndex', "LeftShoulderModel2":'kFBSkeletonLeftShoulderIndex',
           "LeftElbowModel1":'kFBSkeletonLeftElbowIndex', "LeftElbowModel2":'kFBSkeletonLeftElbowIndex', 
           "LeftWristModel1":'kFBSkeletonLeftWristIndex'}            

for key in keyValueList:
    #marker is key
    marker = key
    #bodyId is value
    bodyId = keyValueList[key]
    addMarkerToActor(myMarkerSet, marker, bodyId)


#Those next few lines demonstrate some of the properties that can be modified on the MarkerSet
#set the blend value for a finger. Would be useful if we had fingers...
myActor.LeftHandIndexMiddle = 32

#set the marker translation offset
myMarkerSet.SetMarkerTOffset(FBSkeletonNodeId.kFBSkeletonLeftIndexCIndex, 1, FBVector3d(32, 14, 17))

#hide markerset
myActor.MarkerSet.SetMarkerSetVisibility(False)

#scale the right arm of the actor
scaleValue = FBVector3d(1.1, 1, 1)
myActor.SetDefinitionScaleVector(FBSkeletonNodeId.kFBSkeletonRightElbowIndex, scaleValue)

#translate the actor to the location of the imported motion-capture data
myActor.SetActorTranslation(FBVector3d(-50, 97, -60))

#snap the actor position and activate the actor
myActor.Snap(FBRecalcMarkerSetOffset.kFBRecalcMarkerSetOffsetTR)