#***************************************************************************
# Description
#
# 1. Call FBFolderPopup which allows you to select the folding of which you would like to File > Batch
#
# 2. If there are characters in your current scene, you can set the batch options and plot options for the directory you previously selected
#
# 3. Completes Batch
#
# 4. Saves new file with new name
#
# **One note the folder path is hardcoded currently
#***************************************************************************

from pyfbsdk import FBFolderPopup, FBFilePopupStyle, FBMessageBox, FBSystem, FBApplication, FBBatchStatus, FBBatchOptions, FBBatchFileFormat, FBPlotOptions

# Creating the Constructors
lSystem = FBSystem()
lScene = lSystem.Scene
lApplication = FBApplication ()

# Create the popup and set necessary initial values.
lFp = FBFolderPopup()
lFp.Caption = "Please select the folder you would to use:"

# Set the default path. Good for a PC only... will have to be different for Mac.
lFp.Path = r"C:\Autodesk"

# Get the GUI to show.
lRes = lFp.Execute()


# If we select a folder, show its name, otherwise indicate that the selection was canceled.
if not lRes:
    FBMessageBox( "Warning:", "Selection canceled, cannot continue!", "OK" )
    print "Selection canceled, cannot continue."

else:

    # Setting Batch Options

    # Verify there is a character in the scene
    if not lScene.Characters:
        FBMessageBox( "Error:", "There is no characters in the scene!", "OK" )
        print "Cannot continue at there is not characters in the scene..." 
    else:
        
        # Setting all the plot options

        # Creating FBPlotOptions Constructor
        lPlotOptions = FBPlotOptions()
        
        # Setting all the boolean values for the plot
        lPlotOptions.PlotPeriod.SetSecondDouble  = ( 1.0 / 24.0 )
        lRotationFilter = lPlotOptions.RotationFilterToApply.kFBRotationFilterNone
        lPlotOptions.PlotAllTakes = False
        lPlotOptions.PlotOnFrame = True
        lPlotOptions.UseConstantkeyReducer = False
        lPlotOptions.PlotTranslationOnRootOnly = False
        lPlotOptions.PreciseTimeDiscontinuities    = False

        # Setting all the batch options
        
        # Creating FBBatchOptions Constructor
        lBatchOptions = FBBatchOptions()

        # Look at all the characters in the scene.
        lBatchOptions.Character = lScene.Characters[0]

        # Setting the InputFile Format to be FBX Kaydarea Animation Only
        lFileFormat = lBatchOptions.InputFileFormat.kFBBatchFileFormatFBX
        
        # Setting the input and output directory path
        lBatchOptions.InputDirectory = lFp.Path
        lBatchOptions.OutputDirectory = lFp.Path
       
        # Set load  type
        lType = lBatchOptions.ProcessType.kFBBatchProcessTypeLoad

        # Setting all the boolean values for the batch
        lBatchOptions.FrameAnimation = True
        lBatchOptions.KeepCharacterConstraint = False
        lBatchOptions.KeepDummyBones = False
        lBatchOptions.OverwriteScaling = False
        lBatchOptions.WriteTranslation = False
        lBatchOptions.WriteRate = False
        lBatchOptions.UseSingleTake = False
        lBatchOptions.UseBatchSuffix = False
        lBatchOptions.StartAnimationAtZero = False
        lBatchOptions.SkeletonFile = ""
        lBatchOptions.PlotToControlSet = False
        lBatchOptions.PlotToCharacter = True

        # Start Batch Process
        lBatchStatus = FBBatchStatus()
        lStatus = lApplication.FileBatch( lBatchOptions, lPlotOptions )
        print lStatus


    # Rename Primary File
    saving = FBApplication()
    saving.FileSave("New.fbx")

    # Clean-up
    del (FBApplication, FBBatchOptions)

# Clean-up
del( lFp, lRes, FBFolderPopup, FBFilePopupStyle, FBMessageBox, FBSystem )


'''The Batch functionality lets you convert multiple data files between different formats. You can batch load a series of data files onto the current character, or save all your takes to a specified data format. You can also save all takes mapped to a selected Actor, Character, or Control rig as separate motion files, and convert files from one format to another. 

Before using the Batch window, you must have a character with mapped data from one of the data files that you want to load, save, or convert. All data files or takes must be from the same capture session, and from the same device. 
When you import a motion file and map its motion to an Actor and then to a character, you are creating a guide for converting the other data files. 
For example, if you have four motion files stored in Motion Analysis .trc format to convert to Acclaim .amc format, you can manually convert each file by loading each .trc motion file, mapping the data to an Actor and then a character, plotting the data to the character, then exporting each character as an .amc file. 
The Batch window simplifies this process by letting you base all conversions on a single mapped character. To convert the same four .trc files previously mentioned, you only need to load the first .trc file, map its data to an Actor, then to a Character. You can use the Character as a guide to automatically convert the remaining motion files to .amc format. 
'''
